﻿// --------------------------------------------------------------------------------------
//
// CrossRoads.CS                                                   Ulrich Müller 18.07.09
//                                                                 Aspire        C#  2005
// Assembly CrossRoads mit Funktionen zum Betrieb der education Line CrossRoads
//                       Basis eduLine.DLL v2.09.0.01 / FtLib.LIB v1.74(Knobloch)
// Klasse CrossRoads    : Kapselung und Erweiterung der Funktionen von umFish40.DLL
// Klasse CrossRoadsException : throw CrosRoads Ausnahmen
//
// --------------------------------------------------------------------------------------

using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Text;

namespace CrossRoads40 {
  /// <summary>
  /// Aufzählung der eduLine-Typen
  /// </summary>
  public enum IFTypen {
    /// <summary>
    /// Erstes Device an USB
    /// </summary>
    first_USB = 0,
    /// <summary>
    /// eduLine mit ROBO Interface Typ-Kennung
    /// </summary>
    ROBO_Interface_USB = 60,
    /// <summary>
    /// Allgemein eduLine
    /// </summary>
    EDU_LINE_Module = 300,
    /// <summary>
    /// CrossRoads Device
    /// </summary>
    EDU_LINE_CrossRoads = 301,
    /// <summary>
    /// TrafficLights Device
    /// </summary>
    EDU_LINE_TrafficLights = 302,
    /// <summary>
    /// Einfaches Signal Device
    /// </summary>
    EDU_LINE_Signal = 303
  }
  /// <summary>
  /// Aufzählung der möglichen Schaltungen bei SetLamp
  /// </summary>
  public enum Dir {
    Aus = 0, Off = Aus,
    Ein = 1, On = Ein
  }
  /// <summary>
  /// Aufzählung der Eingänge von CrossRoads
  /// </summary>
  public enum Inp {
    A1 = 1,
    IF1I1 = 1, IF1I2, IF1I3, IF1I4, IF1I5, IF1I6, IF1I7, IF1I8,
    EM1I1, EM1I2, EM1I3, EM1I4, EM1I5, EM1I6, EM1I7, EM1I8
  }
  /// <summary>
  /// Aufzählung der Ausgänge von CrossRoads
  /// </summary>
  public enum Out {
    IF1O1 = 1, IF1O2, IF1O3, IF1O4, IF1O5, IF1O6, IF1O7, IF1O8,
    EM1O1, EM1O2, EM1O3, EM1O4, EM1O5, EM1O6, EM1O7, EM1O8,
    EM2O1, EM2O2, EM2O3, EM2O4, EM2O5, EM2O6, EM2O7, EM2O8,
    EM3O1, EM3O2, EM3O3, EM3O4, EM3O5, EM3O6, EM3O7, EM3O8
  }
  public enum OutM : uint {
    IF1O1 = 0x1, IF1O2 = 0x2, IF1O3 = 0x4, IF1O4 = 0x8,
    IF1O5 = 0x10, IF1O6 = 0x20, IF1O7 = 0x40, IF1O8 = 0x80,
    EM1O1 = 0x100, EM1O2 = 0x200, EM1O3 = 0x400, EM1O4 = 0x800,
    EM1O5 = 0x1000, EM1O6 = 0x2000, EM1O7 = 0x4000, EM1O8 = 0x8000,
    EM2O1 = 0x10000, EM2O2 = 0x20000, EM2O3 = 0x40000, EM2O4 = 0x80000,
    EM2O5 = 0x100000, EM2O6 = 0x200000, EM2O7 = 0x400000, EM2O8 = 0x800000,
    EM3O1 = 0x1000000, EM3O2 = 0x2000000, EM3O3 = 0x4000000, EM3O4 = 0x8000000,
    EM3O5 = 0x10000000, EM3O6 = 0x20000000, EM3O7 = 0x40000000, EM3O8 = 0x80000000
  }
  /// <summary>
  /// Eigenschaften von CrossRoads
  /// </summary>
  public struct DeviceData {
    public string Name;
    public int Type;
    public int SerialNr;
    public string Firmware;
    public override string ToString() {
      return String.Format("{0} ({1}/{2}) v{3}", Name, Type, SerialNr, Firmware);
    }
  }

  /// <summary>
  /// Allgemeine Exception von CrossRoads
  /// </summary>
  public class CrossRoadsException : ApplicationException {
    /// <summary>
    /// Allgemeine CrossLights-Ausnahme
    /// </summary>
    public CrossRoadsException() {
    }
    /// <summary>
    /// Allgemeine CrossLights-Ausnahme
    /// </summary>
    /// <param name="message">Ausnahme-Parameter</param>
    public CrossRoadsException(string message)
      : base(message) {
    }
    /// <summary>
    /// Allgemeine CrossLights-Ausnahme
    /// </summary>
    /// <param name="message">Ausnahme-Parameter</param>
    /// <param name="inner"></param>
    public CrossRoadsException(string message, Exception inner)
      : base(message, inner) {
    }
  }
  
  /// <summary>
  /// Klasse zum Betrieb der eduLine-Devices auf Basis eduLine.DLL
  /// </summary>
  public class CrossRoads {
    // --- eduLine.DLL Deklarationen ----------------------------------------
    [DllImport("eduLine.DLL")]
    public static extern uint edOpenUSB(int ifTyp, int SerialNr);
    [DllImport("eduLine.DLL")]
    public static extern uint edCloseUSB(uint iHandle);

    [DllImport("eduLine.DLL")]
    public static extern int edGetInput(uint iHandle, int InputNr);
    [DllImport("eduLine.DLL")]
    public static extern int edGetInputs(uint iHandle);
    [DllImport("eduLine.DLL")]
    public static extern int edGetVoltage(uint iHandle, int VoltNr);

    [DllImport("eduLine.DLL")]
    public static extern uint edGetOutputs(uint iHandle);
    [DllImport("eduLine.DLL")]
    public static extern uint edSetOutputs(uint iHandle, uint MotorStatus);
    [DllImport("eduLine.DLL")]
    public static extern uint edSetOutput(uint iHandle, int LampNr, int OnOff);

    [DllImport("eduLine.DLL")]
    public static extern uint edGetActDeviceType(uint iHandle);
    [DllImport("eduLine.DLL")]
    public static extern uint edGetActDeviceSerialNr(uint iHandle);
    [DllImport("eduLine.DLL")]
    public static extern uint edGetActDeviceFirmwareNr(uint iHandle);

    public const uint edFehler = 0xE0000001;

    // --- Win API Deklarationen ----------------------------------------------------

    [DllImport("kernel32")]
    public static extern void Sleep(uint mSek);
    [DllImport("kernel32")]
    public static extern uint GetTickCount();
    [DllImport("user32")]
    public static extern uint GetAsyncKeyState(uint vKey);
    public const uint ESCAPE = 0x1B;

        // --- private (globale) Variable ----------------------------------------

    private uint iHandle;        // --- Handle auf aktive umFish40-Instanz
    private bool FOpen;          // --- OpenInterface erfolgreich
    protected bool FNotHalt;       // --- NotHalt
    private bool FDoEvents;      // --- mit/ohne Application.DoEvents

    public const uint rbFehler = 0xE0000001;
    protected const uint rbPollInterval = 10;

    // --- CrossLights : Constructor ---------------------------------------------

    /// <summary>
    /// Konstruktor, parameterlos
    /// </summary>
    public CrossRoads() {
      FOpen = false;
      FNotHalt = false;
    }
    /// <summary>
    /// Löschen aller Ausgänge
    /// </summary>
    /// <exception cref="CrossRoadsException">KeinOpen, InterfaceProblem</exception> 
    public void ClearOutputs() {
      if (!FOpen) throw (new CrossRoadsException("KeinOpen.ClearOutputs"));
      if (edSetOutputs(iHandle, 0) == rbFehler)
        throw (new CrossRoadsException("InterfaceProblem.ClearMotors"));
    }

    /// <summary>
    /// Schließen der Verbindung zu CrossRoads
    /// </summary>
    /// <exception cref="CrossRoadsException">InterfaceProblem</exception>
    public void CloseUSB() {
      if (FOpen) if (edCloseUSB(iHandle) == rbFehler)
          throw (new CrossRoadsException("InterfaceProblem.CloseUSB"));
      FOpen = false;
    }

    /// <summary>
    /// Feststellen Endewunsch durch Escape-Taste oder NotHalt
    /// </summary>
    /// <returns>true bei Endewunsch</returns>
    /// <exception cref="CrossRoadsException">KeinOpen, InterfaceProblem</exception>
    public bool Finish() {
      if (FDoEvents) Application.DoEvents();
      if (!FOpen) throw (new CrossRoadsException("KeinOpen.Finish"));
      if (FNotHalt || GetAsyncKeyState(ESCAPE) != 0) return true;
      if ((uint)GetInputs() == rbFehler)
        throw (new CrossRoadsException("InterfaceProblem.Finish"));
      return false;
    }

    /// <summary>
    /// Auslesen der CrossRoads-Daten
    /// </summary>
    public DeviceData ActDevice {
      get {
        DeviceData dd;
        dd.Type = (int)edGetActDeviceType(iHandle);
        dd.SerialNr = (int)edGetActDeviceSerialNr(iHandle);
        uint f = edGetActDeviceFirmwareNr(iHandle);
        dd.Firmware = String.Format("{0}.{1:00}.{2:00}.{3:00}",
          (f & 0xFF000000) >> 24, (f & 0xFF0000) >> 16,
          (f & 0xFF00) >> 8, (f & 0xFF));
        dd.Name = Convert.ToString((IFTypen)dd.Type);
        return dd;
      }
    }

    /// <summary>
    /// Abfragen des Status eines I-Einganges
    /// </summary>
    /// <param name="InputNr">Name des I-Einganges</param>
    /// <returns>true bei geschlossen</returns>
    /// <exception cref="CrossRoadsException">KeinOpen, InterfaceProblem</exception>
    /// <remarks>Unterbrechbar</remarks>
    public bool GetInput(Inp InputNr) {
      return GetInput((int)InputNr);
    }
    /// <summary>
    /// Abfragen des Status eines I-Einganges
    /// </summary>
    /// <param name="InputNr">Nummer des I-Einganges</param>
    /// <returns>true bei geschlossen</returns>
    /// <exception cref="CrossRoadsException">KeinOpen, InterfaceProblem</exception>
    /// <remarks>Unterbrechbar</remarks>
    public bool GetInput(int InputNr) {
      if (FDoEvents) Application.DoEvents();
      if (!FOpen) throw (new CrossRoadsException("KeinOpen.GetInput"));
      int res = edGetInput(iHandle, InputNr);
      if ((uint)res == rbFehler)
        throw (new CrossRoadsException("InterfaceProblem.GetInput"));
      return (res == 1) ? true : false;
    }

    /// <summary>
    /// Auslesen des Status aller I-Eingänge
    /// </summary>
    /// <returns>bitWerte aller I-Eingänge, einschl. angeschlossener Extensions</returns>
    /// <exception cref="CrossRoadsException">KeinOpen, InterfaceProblem</exception>
    /// <remarks>Unterbrechbar</remarks>
    public int GetInputs() {
      if (!FOpen) throw (new CrossRoadsException("KeinOpen.GetInputs"));
      if (FDoEvents) Application.DoEvents();
      int res = edGetInputs(iHandle);
      if ((uint)res == rbFehler)
        throw (new CrossRoadsException("InterfaceProblem.GetInputs"));
      return res;
    }

    /// <summary>
    /// Auslesen eines Spannungswertes
    /// </summary>
    /// <param name="VoltNr">Name des Spannungseinganges</param>
    /// <returns>Spannungswert in 1/100 Volt</returns>
    /// <exception cref="CrossRoadsException">KeinOpen, InterfaceProblem</exception>
    /// <remarks>Unterbrechbar</remarks>
    public int GetVoltage(Inp VoltNr) {
      return GetVoltage((int)VoltNr);
    }
    /// <summary>
    /// Auslesen eines Spannungswertes
    /// </summary>
    /// <param name="VoltNr">Nummer des Spannungseinganges</param>
    /// <returns>Spannungswert in 1/100 Volt</returns>
    /// <exception cref="CrossRoadsException">KeinOpen, InterfaceProblem</exception>
    /// <remarks>Unterbrechbar</remarks>
    public int GetVoltage(int VoltNr) {
      if (FDoEvents) Application.DoEvents();
      if (!FOpen) throw (new CrossRoadsException("KeinOpen.GetVoltage"));
      int res = edGetVoltage(iHandle, VoltNr);
      if ((uint)res == rbFehler)
        throw (new CrossRoadsException("InterfaceProblem.GetVoltage"));
      return res;
    }

    /// <summary>
    /// Abbruchwunsch
    /// </summary>
    /// 
    public bool NotHalt {
      get {
        return FNotHalt;
      }
      set {
        FNotHalt = value;
      }
    }

    /// <summary>
    /// Verbindung zum ersten eduLine-Device an USB herstellen
    /// </summary>
    public void OpenUSB() {
      OpenUSB(0, 0, true);
    }
    /// <summary>
    /// Verbindung zu eduLine-Device herstellen
    /// </summary>
    /// <param name="ifTyp">Name des zutreffenden eduLine-Devices</param>
    /// <param name="SerialNr">
    /// Aktive Seriennummer, bei ROBO first USB stets 0</param>
    /// <exception cref="CrossRoadsException">InterfaceProblem</exception>
    public void OpenUSB(IFTypen ifTyp, int SerialNr) {
      OpenUSB(ifTyp, SerialNr, true);
    }
    /// <summary>
    /// Verbindung zu einem eduLine-Device an USB herstellen
    /// </summary>
    /// <param name="ifTyp">Name des zutreffenden eduLine-Device</param>
    /// <param name="SerialNr">
    /// Aktive Seriennummer, bei ROBO first USB stets 0</param>
    /// <param name="DoEvents"> Unterbrechbar</param>
    /// <exception cref="CrossRoadsException">InterfaceProblem</exception>
    public void OpenUSB(IFTypen ifTyp, int SerialNr, bool DoEvents) {
      iHandle = edOpenUSB((int)ifTyp, SerialNr);
      if ((uint)iHandle == rbFehler)
        throw (new CrossRoadsException("DeviceProblem.Open"));
      edSetOutputs(iHandle, 0);
      FNotHalt = false;
      FOpen = true;
      FDoEvents = DoEvents;
    }

    /// <summary>
    /// Ausgabe OpenStatus
    /// </summary>
    public bool OpenStatus {
      get {
        return FOpen;
      }
    }

    /// <summary>
    /// Auslesen aller Ausgänge
    /// </summary>
    /// <exception cref="CrossRoadsException">KeinOpen, InterfaceProblem</exception>
    public uint Outputs {
      get {
        if (!FOpen) throw (new CrossRoadsException("KeinOpen.Outputs"));
        uint res = edGetOutputs(iHandle);
        if ((uint)res == rbFehler)
          throw (new CrossRoadsException("DeviceProblem.Outputs"));
        return res;
      }
    }

    /// <summary>
    /// Programmablauf anhalten
    /// </summary>
    /// <param name="mSek">Pausendauer in Millisekunden</param>
    /// <exception cref="CrossRoadsException">KeinOpen, DeviceProblem</exception>
    /// <remarks>Unterbrechbar, Abbrechbar</remarks>
    public void Pause(int mSek) {
      if (!FOpen) throw (new CrossRoadsException("KeinOpen.Pause"));
      uint sZeit = GetTickCount();
      do {
        if (FDoEvents) Application.DoEvents();
        if ((uint)GetInputs() == rbFehler)
          throw (new CrossRoadsException("DeviceProblem.Pause"));
        if (FNotHalt || GetAsyncKeyState(ESCAPE) != 0) return;
        Sleep(rbPollInterval);
      } while (GetTickCount() - sZeit < mSek);
    }

    // --- SetOutput : Setzen eines O-Ausganges.

    /// <summary>
    /// Setzen eines O-Ausganges
    /// </summary>
    /// <param name="LampNr">Name des O-Ausganges</param>
    /// <param name="OnOff">Ein- oder Ausschalten</param>
    /// <exception cref="CrossRoadsException">KeinOpen, DeviceProblem</exception>
    /// <remarks>Unterbrechbar</remarks>
    public void SetOutput(Out OutputNr, Dir OnOff) {
      SetOutput((int)OutputNr, (int)OnOff);
    }
    /// <summary>
    /// Setzen eines O-Ausganges
    /// </summary>
    /// <param name="LampNr">Nummer des O-Ausganges</param>
    /// <param name="OnOff">Ein- oder Ausschalten (1/0)</param>
    /// <exception cref="CrossLightsException">KeinOpen, DeviceProblem</exception>
    /// <remarks>Unterbrechbar</remarks>
    public void SetOutput(int OutputNr, int OnOff) {
      if (FDoEvents) Application.DoEvents();
      if (!FOpen) throw (new CrossRoadsException("KeinOpen.SetLamp"));
      if (edSetOutput(iHandle, OutputNr, OnOff) == rbFehler)
        throw (new CrossRoadsException("DeviceProblem.SetLamp"));
    }

    /// <summary>
    /// Setzen aller Ausgänge
    /// </summary>
    /// <param name="Status">
    /// Schaltzustand aller angeschlossenen Augänge</param>
    /// <exception cref="CrossRoadsException">KeinOpen, DeviceProblem</exception>
    /// <remarks>Unterbrechbar</remarks>
    public void SetOutputs(uint Status) {
      if (FDoEvents) Application.DoEvents();
      if (!FOpen) throw (new CrossRoadsException("KeinOpen.SetOutputs"));
      if (edSetOutputs(iHandle, Status) == rbFehler)
        throw (new CrossRoadsException("DeviceProblem.SetOutputs"));
    }

    /// <summary>
    /// Warten auf I-Eingang = true
    /// </summary>
    /// <param name="InputNr">Name des I-Einganges</param>
    /// <exception cref="CrossRoadsException">KeinOpen, InterfaceProblem</exception>
    /// <remarks>Unterbrechbar, Abbrechbar</remarks>
    public void WaitForInput(Inp InputNr) {
      WaitForInput((int)InputNr);
    }
    /// <summary>
    /// Warten auf I-Eingang = true
    /// </summary>
    /// <param name="InputNr">Nummer des I-Einganges</param>
    /// <exception cref="CrossLightsException">KeinOpen, DeviceProblem</exception>
    /// <remarks>Unterbrechbar, Abbrechbar</remarks>
    public void WaitForInput(int InputNr) {
      WaitForInput(InputNr, true);
    }
    /// <summary>
    /// Warten auf I-Eingang = OnOff
    /// </summary>
    /// <param name="InputNr">Name des I-Einganges</param>
    /// <param name="OnOff">true/false</param>
    /// <exception cref="CrossRoadsException">KeinOpen, DeviceProblem</exception>
    /// <remarks>Unterbrechbar, Abbrechbar</remarks>
    public void WaitForInput(Inp InputNr, bool OnOff) {
      WaitForInput((int)InputNr, OnOff);
    }
    /// <summary>
    /// Warten auf I-Eingang = OnOff
    /// </summary>
    /// <param name="InputNr">Name des I-Einganges</param>
    /// <param name="OnOff">true/false</param>
    /// <exception cref="CrossRoadsException">KeinOpen, InterfaceProblem</exception>
    /// <remarks>Unterbrechbar, Abbrechbar</remarks>
    public void WaitForInput(int InputNr, bool OnOff) {
      if (!FOpen) throw (new CrossRoadsException("KeinOpen.WaitForInput"));
      do {
        if (FNotHalt || GetAsyncKeyState(ESCAPE) != 0) return;
        if ((uint)GetInputs() == rbFehler)
          throw (new CrossRoadsException("DeviceProblem.WaitForInput"));
        Sleep(rbPollInterval);
        if (FDoEvents) Application.DoEvents();
      } while (!GetInput(InputNr) == OnOff);
    }

    /// <summary>
    /// Versionsabfrage
    /// </summary>
    public static string Version {
      get {
        return "2.09.00.01";
      }
    }

  }
}
