﻿# -----------------------------------------------------------------------------
#
# eduCrossRoads.py                                  Ulrich Mueller 13.12.11
#                                                   PCS1700    Python 3.1.1
# Modul eduCrossRoads mit Funktionen zum Betrieb der education Line Devices
# Klasse CrossRoads          : Kapselung und Erweiterung der Funktionen 
#                              von umFish40.DLL  v4.77
# Klasse CrossRoadsException : Raise CrossRoad Ausnahmen
#
# -----------------------------------------------------------------------------

from ctypes import *

class CrossRoadsException(IOError):
  "Allgemeiner CrossRoad Fehler"

class CrossRoads(object):

  iHandle      = -1          # Handle für das aktive Interfacee
                             # gleichzeitig Open-Merker (no Open = -1)
  PollInterval = 12
  
# ---- symbolische Konstanten -------------------
  Fehler       = 0x1FFFF
  Ein          = 1
  Aus          = 0

  first_USB              =   0
  ROBO_Interface_USB     =  60
  EDU_Line_Module        =  300
  EDU_Line_CrossRoads    =  301
  EDU_Line_TrafficLights =  302
  EDU_Line_Signal        =  303
  
# --- interne Werte ---------------------------
  ESCAPE       = 27
  
  def __init__(self):
    self.iHandle = -1
    
# --- öffentliche Methoden und Eigenschaften ---

  def OpenUSB(self, TypNr=0, Nr=0):
    "Verbindung zum education Line Device herstellen"
    self.iHandle = windll.umFish40.rbOpenInterfaceUSB(TypNr, Nr)
    print ("USB", TypNr, Nr)
    if self.iHandle == self.Fehler:
      raise (CrossRoadsException, "DeviceProblem.Open")  
    self.SetOutputs(0)        

  def ClearOutputs(self):
    "Loeschen aller Ausgaenge"
    windll.umFish40.edSetOutputs(self.iHandle, 0)
    
  def CloseUSB(self):
    "Schliessen education Line Device"
    windll.umFish40.rbCloseInterface(self.iHandle)
    self.iHandle = -1

  def Finish(self, InputNr=0):
    "Feststellen Endewunsch (Escape)"
    if windll.user32.GetAsyncKeyState(self.ESCAPE) != 0: return 1
    return 0
    
  def GetInput(self, InputNr):
    "Bestimmen Status Eingang"
    return windll.umFish40.rbGetInput(self.iHandle, InputNr)
  
  def GetInputs(self):
    "Auslesen Status aller Eingaenge"
    return windll.umFish40.rbGetInputs(self.iHandle)

  def GetOutputs(self):
    "Auslesen Status aller Ausgaenge"
    return windll.umFish40.rbGetMotors(self.iHandle)

  def GetVoltage(self, VoltNr):
    "Auslesen Voltwert A1"
    return windll.umFish40.rbGetVoltage(self.iHandle, VoltNr)
  
  def Pause(self, mSek):
    "Programm anhalten"    
    sZeit = windll.kernel32.GetTickCount()
    while (windll.kernel32.GetTickCount() - sZeit) < mSek:
      if windll.user32.GetAsyncKeyState(self.ESCAPE) != 0 : return
      windll.kernel32.Sleep(self.PollInterval)

  def SetOutput(self, LampNr, OnOff):
    "Setzen eines Ausganges"
    windll.umFish40.rbSetLamp(self.iHandle, LampNr, OnOff)
    
  def SetOutputs(self, Wert):
    "Setzen aller Ausgaenge"
    windll.umFish40.rbSetMotors(self.iHandle, Wert)
  
  def Version(self):
    "Version der Klasse"
    return "4.77.00.01"

  def WaitForInput(self, InputNr, OnOff=1):
    "Warten auf InputNr = OnOff"
    while not (self.GetInput(InputNr) == OnOff):
      windll.kernel32.Sleep(self.PollInterval)
      if windll.user32.GetAsyncKeyState(self.ESCAPE) != 0 : return
