// -----------------------------------------------------------------------------
//
// eduLine.CPP                                            Ulrich Mller 16.07.09
//                                                        PCS1700       VC++ 6.0
// Basisfunktionen fr die Devices der Knobloch education Line unter Nutzung
//                 der FtLib_Static-LIBCMT_...lib
//
// Fehler-Return allgemein rbFehler, OK = 0 bzw. def. Rckgabewert < 0xE0000000
//
// ------------------------------------------------------------------------------

#include <Windows.h>
#include <iostream.h>
#include "FtLib.h"
#include "eduLine.H"

// ----------- globale Konstanten ----------------------------------------------

const int rbFehler = 0xE0000001L;

// ----------- globale Variable ------------------------------------------------

RoboInstanz rbI[8];

// OpenInterfaceAll ------------------------------------------------------------
//
// Setzen Ausgangswerte, Starten TransferArea
//
// ------------------------------------------------------------------------------

int OpenInterfaceAll(FT_HANDLE ftHandle) {
	int i;
	// --- Suchen eines freien RoboInstanz-Eintrage in rbI ----
  int iHandle = rbFehler;
	for(i = 0; i < 8; i++)
		if(rbI[i].ftHandle == 0) {
			iHandle = i; break; }
		if(iHandle == rbFehler) return rbFehler;

	rbI[iHandle].ftHandle = ftHandle;

	if(StartFtTransferArea(rbI[iHandle].ftHandle, NULL))
		return rbFehler;
	rbI[iHandle].ftDCB = GetFtTransferAreaAddress(rbI[iHandle].ftHandle);
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;

	// --- weiteres Besetzen RoboInstanz ------

	rbI[iHandle].DistanceMode = 0;
	
	for(i = 0; i < 32; i++) rbI[iHandle].iCounter[i] = 0;
	rbI[iHandle].mModeStatus = 0;
	return iHandle;
}

// edOpenUSB ----------------------------------------------------------
//
// ffnen eines education Line Devices ber einen USB-Anschlu
//
// -----------------------------------------------------------------------------

int __stdcall edOpenUSB(int ifTyp, int SerialNr) {
	FT_HANDLE usbHandle;
	if (IsFtLibInit() == FTLIB_ERR_LIB_IS_NOT_INITIALIZED) {
		if(InitFtLib()) return rbFehler;
	}
	DWORD res = InitFtUsbDeviceList();
	if(!(res == 0 || res == FTLIB_ERR_SOME_DEVICES_ARE_OPEN)) return rbFehler;
	UINT anzUSBDev = GetNumFtUsbDevice();
	if(!anzUSBDev) return rbFehler;
	if(ifTyp == 0) usbHandle = GetFtUsbDeviceHandle(0);
	else usbHandle = GetFtUsbDeviceHandleSerialNr(SerialNr, ifTyp);
	if(!usbHandle) return rbFehler;

	if(OpenFtUsbDevice(usbHandle)) {return rbFehler;}
	return OpenInterfaceAll(usbHandle);
}


// edCloseClose -------------------------------------------------------------
//
// Stop TransferArea, Schlieen Schnittstelle
//
// ------------------------------------------------------------------------------

int __stdcall edCloseUSB(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(StopFtTransferArea(rbI[iHandle].ftHandle)) return rbFehler;
	if(CloseFtDevice(rbI[iHandle].ftHandle)) return rbFehler;
	rbI[iHandle].ftDCB = NULL;
	rbI[iHandle].ftHandle = 0;
	return 0;
}

// edGetInput -------------------------------------------------------------------
//
// Abfrage eines I-Einganges
//
// ------------------------------------------------------------------------------

int __stdcall edGetInput(int iHandle, int InputNr) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	InputNr--;
	DWORD c = InputNr >> 3;
	DWORD n = InputNr % 8;
	switch(c) {
		case 0:  return (rbI[iHandle].ftDCB->E_Main & EMaske[n]) ? 1 : 0; break;
		case 1:	 return (rbI[iHandle].ftDCB->E_Sub1 & EMaske[n]) ? 1 : 0; break;
		case 2:	 return (rbI[iHandle].ftDCB->E_Sub2 & EMaske[n]) ? 1 : 0; break;
		case 3:	 return (rbI[iHandle].ftDCB->E_Sub3 & EMaske[n]) ? 1 : 0; break;
		default: return rbFehler;
	}
}

// edGetInputs ------------------------------------------------------------------
//
// Status aller I-Eingnge
//
// ------------------------------------------------------------------------------

int __stdcall edGetInputs(int iHandle) {
	DWORD InputStatus = 0;
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	InputStatus  = rbI[iHandle].ftDCB->E_Main;
	InputStatus |= rbI[iHandle].ftDCB->E_Sub1 << 8;
	InputStatus |= rbI[iHandle].ftDCB->E_Sub2 << 16;
	InputStatus |= rbI[iHandle].ftDCB->E_Sub3 << 24;
	return InputStatus;
}

// edGetVoltage -----------------------------------------------------------------
//
// Abfrage eines A1-Einganges
//
// ------------------------------------------------------------------------------

int __stdcall edGetVoltage(int iHandle, int VoltNr) {

	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	switch (VoltNr) {
		case ftiA1:  return rbI[iHandle].ftDCB->A1; break;
		case ftiA2:  return rbI[iHandle].ftDCB->A2; break;
		case ftiAV:  return rbI[iHandle].ftDCB->AV; break;
		case ftiAZ:  return rbI[iHandle].ftDCB->AZ; break;
		case ftiD1V: return rbI[iHandle].ftDCB->D1; break;
		case ftiD2V: return rbI[iHandle].ftDCB->D2; break;
		default:     return 0;
	}
}

// edGetOutputs ----------------------------------------------------------------
//
// (Direction) Status aller O-Ausgnge
//
// ----------------------------------------------------------------------------

UINT __stdcall edGetOutputs(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	DWORD MotorStatus = 0;
	MotorStatus  = rbI[iHandle].ftDCB->M_Main;
	MotorStatus |= rbI[iHandle].ftDCB->M_Sub1 << 8;
	MotorStatus |= rbI[iHandle].ftDCB->M_Sub2 << 16;
	MotorStatus |= rbI[iHandle].ftDCB->M_Sub3 << 24;
	return MotorStatus;
}

// edSetOutputs ----------------------------------------------------------------
//
// Setzen des (Direction) Status aller M-Ausgnge, Speed = 7, Mode = 0
//
// ----------------------------------------------------------------------------

int __stdcall edSetOutputs(int iHandle, UINT MotorStatus) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	rbI[iHandle].ftDCB->M_Main =  MotorStatus & 0x000000FF;
	rbI[iHandle].ftDCB->M_Sub1 = (MotorStatus & 0x0000FF00) >> 8;
	rbI[iHandle].ftDCB->M_Sub2 = (MotorStatus & 0x00FF0000) >> 16;
	rbI[iHandle].ftDCB->M_Sub3 = (MotorStatus & 0xFF000000) >> 24;
	rbI[iHandle].mModeStatus   = 0;
	for(int i = 0; i < 8; i++) {
		rbI[iHandle].ftDCB->MPWM_Main[i] = 7;
		rbI[iHandle].ftDCB->MPWM_Sub1[i] = 7;
		rbI[iHandle].ftDCB->MPWM_Sub2[i] = 7;
		rbI[iHandle].ftDCB->MPWM_Sub3[i] = 7;
	}
	rbI[iHandle].mModeStatus = 0;
	return 0;
}

// edSeOutput ----------------------------------------------------------------
//
// Setzen eines O-Ausganges
//
// ----------------------------------------------------------------------------

int __stdcall edSetOutput(int iHandle, int LampNr, int OnOff) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	LampNr--;
	DWORD c = LampNr >> 3;
	DWORD n = LampNr % 8;
	switch(c) {
		case 0:
			if(OnOff) rbI[iHandle].ftDCB->M_Main |=  EMaske[n];
			else      rbI[iHandle].ftDCB->M_Main &= ~EMaske[n];
			rbI[iHandle].ftDCB->MPWM_Main[n] = 7;
			break;
		case 1:
			if(OnOff) rbI[iHandle].ftDCB->M_Sub1 |=  EMaske[n];
			else      rbI[iHandle].ftDCB->M_Sub1 &= ~EMaske[n];
			rbI[iHandle].ftDCB->MPWM_Sub1[n] = 7;
			break;
		case 2:
			if(OnOff) rbI[iHandle].ftDCB->M_Sub2 |=  EMaske[n];
			else      rbI[iHandle].ftDCB->M_Sub2 &= ~EMaske[n];
			rbI[iHandle].ftDCB->MPWM_Sub2[n] = 7;
			break;
		case 3:
			if(OnOff) rbI[iHandle].ftDCB->M_Sub3 |=  EMaske[n];
			else      rbI[iHandle].ftDCB->M_Sub3 &= ~EMaske[n];
			rbI[iHandle].ftDCB->MPWM_Sub3[n] = 7;
			break;
		default: return rbFehler;
	}
	return 0;
}


// edDevice -----------------------------------------------------------------
//
// Auskunftsfunktionen : edGetActDeviceType, SerialNr, FirmwareNr
//                                           Name,     Firmware
//
// ---------------------------------------------------------------------------
int __stdcall edGetActDeviceType(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	return GetFtDeviceTyp(rbI[iHandle].ftHandle);
}
int __stdcall edGetActDeviceSerialNr(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	return GetFtSerialNr(rbI[iHandle].ftHandle);
}
int __stdcall edGetActDeviceFirmwareNr(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return rbFehler;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return rbFehler;
	return GetFtFirmware(rbI[iHandle].ftHandle);
}
LPCSTR __stdcall edGetActDeviceFirmware(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return NULL;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return NULL;
	return GetFtFirmwareStrg(rbI[iHandle].ftHandle);
}
LPCSTR __stdcall edGetActDeviceName(int iHandle) {
	if(rbI[iHandle].ftDCB == NULL) return NULL;
	if(!(IsFtTransferActiv(rbI[iHandle].ftHandle) == FTLIB_ERR_THREAD_IS_RUNNING)) 
		return NULL;
	return GetFtShortNameStrg(rbI[iHandle].ftHandle);
}

// edSystem -----------------------------------------------------------------
//
// Bereitstellung von Systemfunktionen
//
// --------------------------------------------------------------------------
void __stdcall edSleep(int mSek) {
	Sleep(mSek);
}
int __stdcall edESC(void) {
	return GetAsyncKeyState(VK_ESCAPE);
}

// DllMain -------------------------------------------------------------------
//
// Hauptschleife bei Start und Ende
// Schlieen aller RoboInstanzen, wenn nicht bereits geschehen
// Schlieen FtLib Arbeitsbereich
//
// ---------------------------------------------------------------------------

BOOL WINAPI DllMain(HINSTANCE hDLL, DWORD dwReason, LPVOID) {
	switch(dwReason) {
		case DLL_PROCESS_DETACH:
			for(int i = 0; i < 8; i++) edCloseUSB(i);
			CloseFtLib();
			break;
	}
	return TRUE;
}